var _a, _b;
import mockFetchOn from './core.js';
// Export core package
export * from './core.js';
export { mockFetchOn };
// Lib compatibility checks.
const swSupported = typeof window !== 'undefined' && !!((_a = window === null || window === void 0 ? void 0 : window.navigator) === null || _a === void 0 ? void 0 : _a.serviceWorker);
const isInNode = typeof process !== 'undefined' && ((_b = process === null || process === void 0 ? void 0 : process.release) === null || _b === void 0 ? void 0 : _b.name) === 'node';
// Lib clients. Initialize when needed.
const channelMap = new Map();
let mswInterceptors;
let wpRoute;
// Context helpers.
let contextHelpers = {};
// Mock globalThis and activate by default.
const fetchMock = mockFetchOn(globalThis);
fetchMock.activate();
const onfetch = Object.assign(
// Use a copy to avoid overwriting the fetchMock props in runtime.
fetchMock.bind(fetchMock), fetchMock, {
    async useServiceWorker(serviceWorkerContainer = navigator.serviceWorker) {
        if (!swSupported) {
            throw new Error('Environment not compatible with service worker mode');
        }
        const channel = await (async () => {
            const existing = channelMap.get(serviceWorkerContainer);
            if (existing)
                return existing;
            const Channel = (await import('./lib/SW/Channel.js')).default;
            const newChannel = new Channel(serviceWorkerContainer);
            channelMap.set(serviceWorkerContainer, newChannel);
            return newChannel;
        })();
        await this.adopt(channel, channel);
        fetchMock.config({
            bypassRedirect: true,
        });
    },
    async useMSWInterceptors() {
        if (!mswInterceptors) {
            if (!isInNode) {
                throw new Error('Environment not compatible with msw interceptor mode');
            }
            const MSWInterceptors = (await import('./lib/MSWInterceptors/index.js')).default;
            mswInterceptors = new MSWInterceptors();
        }
        await this.adopt(mswInterceptors, mswInterceptors);
        fetchMock.config({
            bypassRedirect: true,
        });
    },
    async useWrightplayRoute() {
        if (!wpRoute) {
            if (isInNode) {
                throw new Error('Environment not compatible with wrightplay route mode');
            }
            // Intended try-catch for downstream bundlers like esbuild to know the package
            // (wrightplay in this case) is not always required when bundling for browsers.
            // Node.js libs don't need this as they can be safely fully excluded by using
            // package browser field.
            try {
                const wp = await import('wrightplay');
                const WPRoute = (await import('./lib/WPRoute/index.js')).default;
                wpRoute = new WPRoute(wp);
            }
            catch (e) {
                throw new Error('Failed load wrightplay route lib. Have you installed "wrightplay"?', { cause: e });
            }
        }
        await this.adopt(wpRoute, wpRoute);
        fetchMock.config({
            bypassRedirect: false,
        });
    },
    async useAutoAdvanced() {
        if (swSupported) {
            await this.useServiceWorker();
        }
        else if (isInNode) {
            await this.useMSWInterceptors();
        }
        else {
            throw new Error('Environment not compatible with any advanced mode');
        }
    },
    async useDefault() {
        await this.adopt(globalThis);
        fetchMock.config({
            bypassRedirect: false,
        });
    },
    isActive() {
        var _a, _b;
        return (_b = (_a = contextHelpers.isActive) === null || _a === void 0 ? void 0 : _a.call(contextHelpers)) !== null && _b !== void 0 ? _b : fetchMock.isActive();
    },
    async activate() {
        var _a;
        await ((_a = contextHelpers.activate) === null || _a === void 0 ? void 0 : _a.call(contextHelpers));
        fetchMock.activate();
    },
    async restore() {
        var _a;
        fetchMock.restore();
        await ((_a = contextHelpers.restore) === null || _a === void 0 ? void 0 : _a.call(contextHelpers));
    },
    async adopt(context, controllers = {}) {
        var _a, _b;
        const beActive = this.isActive();
        await ((_a = contextHelpers.restore) === null || _a === void 0 ? void 0 : _a.call(contextHelpers));
        contextHelpers = controllers;
        if (beActive)
            await ((_b = contextHelpers.activate) === null || _b === void 0 ? void 0 : _b.call(contextHelpers));
        fetchMock.adopt(context);
    },
});
export default onfetch;
//# sourceMappingURL=index.js.map