import nodeInterceptors from '@mswjs/interceptors/lib/presets/node.js';
import { BatchInterceptor } from '@mswjs/interceptors';
export default class MSWInterceptors {
    constructor() {
        Object.defineProperty(this, "bypassNext", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        /**
         * The fetch that bypasses the mock.
         */
        Object.defineProperty(this, "fetch", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (...args) => {
                this.bypassNext = true;
                return fetch(...args);
            }
        });
        /**
         * The resolver that parses the requests.
         */
        Object.defineProperty(this, "resolver", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: async (mswRequest) => {
                if (this.bypassNext) {
                    this.bypassNext = false;
                    return;
                }
                const request = new Request(mswRequest.url.href, {
                    ...mswRequest,
                    // avoid the invalid '' body in GET/HEAD requests.
                    body: mswRequest.body || null,
                });
                const response = await this.fetch(request);
                const { status, statusText, headers } = response;
                mswRequest.respondWith({
                    status,
                    statusText,
                    headers: [...headers]
                        .reduce((acc, [key, value]) => {
                        if (acc[key]) {
                            acc[key].push(value);
                        }
                        else {
                            acc[key] = [value];
                        }
                        return acc;
                    }, {}),
                    body: await response.text(),
                });
            }
        });
        Object.defineProperty(this, "interceptor", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new BatchInterceptor({
                name: 'onfetch',
                interceptors: nodeInterceptors.default,
            })
        });
        this.interceptor.on('request', this.resolver);
    }
    activate() {
        this.interceptor.apply();
    }
    restore() {
        this.interceptor.dispose();
    }
}
//# sourceMappingURL=index.js.map