import { CloneableRequest, CloneableResponse } from './toCloneable.js';
export interface StatusMessage {
    status: 'on' | 'off';
}
interface NetworkMessage {
    index: number;
}
export declare type FulfillList = ([(res: Response) => void, (err: Error) => void] | null)[];
export interface RequestMessage extends NetworkMessage {
    request: CloneableRequest;
}
export interface ResponseMessage extends NetworkMessage {
    response: CloneableResponse | Error;
}
export declare abstract class MessageProcessor {
    abstract onRequestMessage(event: MessageEvent<RequestMessage>): unknown;
    abstract onResponseMessage(event: MessageEvent<ResponseMessage>): unknown;
    abstract onStatusMessage(event: MessageEvent<StatusMessage>): unknown;
    onMessage: (event: MessageEvent<RequestMessage | ResponseMessage | StatusMessage>) => void;
    abstract isActive(): boolean;
    abstract switchToStatus(status: 'on' | 'off'): Promise<void>;
    /**
     * Start receiving messages.
     */
    activate(): Promise<void>;
    /**
     * Stop receiving messages.
     */
    restore(): Promise<void>;
}
export {};
//# sourceMappingURL=Message.d.ts.map