/// <reference types="serviceworker" />
import { FulfillList, StatusMessage, RequestMessage, ResponseMessage, MessageProcessor } from './Message.js';
/**
 * A fetch forwarder
 * - For captured requests, message back to client.
 * - For messaged requests, message back the real response.
 */
export default class Worker extends MessageProcessor {
    scope: ServiceWorkerGlobalScope;
    portMap: Map<string, MessagePort>;
    fulfillListMap: Map<MessagePort, FulfillList>;
    constructor(scope: ServiceWorkerGlobalScope);
    /**
     * For captured requests, message back to the client.
     */
    onFetch(event: FetchEvent): void;
    /**
     * For messaged requests, respond with real responses.
     */
    onRequestMessage(event: MessageEvent<RequestMessage>): Promise<void>;
    /**
     * For messaged responses, treat as a previous captured request's response.
     */
    onResponseMessage(event: MessageEvent<ResponseMessage>): void;
    statusResolveMap: Map<MessagePort, (() => void) | null>;
    onStatusMessage(event: MessageEvent<StatusMessage>): void;
    isActive(): boolean;
    switchToStatus(status: 'on' | 'off'): Promise<void>;
}
//# sourceMappingURL=Worker.d.ts.map